@@ -2,7 +2,7 @@
  * CoreInteraction.m: MacOS X interface module
  *****************************************************************************
  * Copyright (C) 2011-2012 Felix Paul Kühne
- * $Id: ccd3181a0a582ff98e810ba3647ec2d9f250fa11 $
+ * $Id: 580dd5dc0450350950175a5e5f42de60971ddd23 $
  *
  * Authors: Felix Paul Kühne <fkuehne -at- videolan -dot- org>
  *
@@ -24,6 +24,7 @@
 #import "CoreInteraction.h"
 #import "intf.h"
 #import "open.h"
+#import "playlist.h"
 #import <vlc_playlist.h>
 #import <vlc_input.h>
 #import <vlc_keys.h>
@@ -31,6 +32,7 @@
 #import <vlc_aout_intf.h>
 #import <vlc/vlc.h>
 #import <vlc_strings.h>
+#import <vlc_url.h>
 
 @implementation VLCCoreInteraction
 static VLCCoreInteraction *_o_sharedInstance = nil;
@@ -45,7 +47,7 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
 
 - (id)init
 {
-    if( _o_sharedInstance)
+    if( _o_sharedInstance )
     {
         [self dealloc];
         return _o_sharedInstance;
@@ -53,7 +55,6 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
     else
     {
         _o_sharedInstance = [super init];
-        b_lockAspectRatio = YES;
     }
 
     return _o_sharedInstance;
@@ -119,7 +120,7 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
 - (void)toggleRecord
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return;
 
     input_thread_t * p_input;
@@ -147,12 +148,12 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
     float f_rate;
 
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return 0;
 
     input_thread_t * p_input;
     p_input = pl_CurrentInput( p_intf );
-    if (p_input)
+    if( p_input )
     {
         f_rate = var_GetFloat( p_input, "rate" );
         vlc_object_release( p_input );
@@ -188,18 +189,18 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
 - (BOOL)isPlaying
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return NO;
 
     input_thread_t * p_input = pl_CurrentInput( p_intf );
-
-    if (!p_input) return NO;
+    if( !p_input )
+        return NO;
 
     input_state_e i_state = ERROR_S;
-    input_Control( p_input, INPUT_GET_STATE, &i_state);
+    input_Control( p_input, INPUT_GET_STATE, &i_state );
     vlc_object_release( p_input );
 
-    return ((i_state == OPENING_S) || (i_state == PLAYING_S));
+    return ( ( i_state == OPENING_S ) || ( i_state == PLAYING_S ) );
 }
 
 - (int)currentTime
@@ -207,9 +208,10 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
     input_thread_t * p_input = pl_CurrentInput( VLCIntf );
     int64_t i_currentTime = -1;
 
-    if (!p_input) return i_currentTime;
+    if( !p_input )
+        return i_currentTime;
 
-    input_Control( p_input, INPUT_GET_TIME, &i_currentTime);
+    input_Control( p_input, INPUT_GET_TIME, &i_currentTime );
     vlc_object_release( p_input );
 
     return (int)( i_currentTime / 1000000 );
@@ -220,47 +222,49 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
     int64_t i64_value = (int64_t)i_value;
     input_thread_t * p_input = pl_CurrentInput( VLCIntf );
 
-    if (!p_input) return;
+    if ( !p_input )
+        return;
 
-    input_Control( p_input, INPUT_SET_TIME, (int64_t)(i64_value * 1000000));
+    input_Control( p_input, INPUT_SET_TIME, (int64_t)(i64_value * 1000000) );
     vlc_object_release( p_input );
 }
 
 - (int)durationOfCurrentPlaylistItem
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return 0;
 
     input_thread_t * p_input = pl_CurrentInput( p_intf );
     int64_t i_duration = -1;
-    if (!p_input) return i_duration;
+    if( !p_input )
+        return i_duration;
 
-
-    input_Control( p_input, INPUT_GET_LENGTH, &i_duration);
+    input_Control( p_input, INPUT_GET_LENGTH, &i_duration );
     vlc_object_release( p_input );
 
-    return (int)(i_duration / 1000000);
+    return (int)( i_duration / 1000000 );
 }
 
 - (NSURL*)URLOfCurrentPlaylistItem
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return nil;
 
     input_thread_t *p_input = pl_CurrentInput( p_intf );
-    if (!p_input) return nil;
+    if( !p_input )
+        return nil;
 
     input_item_t *p_item = input_GetItem( p_input );
-    if (!p_item)
+    if( !p_item )
     {
         vlc_object_release( p_input );
         return nil;
     }
 
     char *psz_uri = input_item_GetURI( p_item );
-    if (!psz_uri)
+    if( !psz_uri )
     {
         vlc_object_release( p_input );
         return nil;
@@ -276,21 +280,22 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
 - (NSString*)nameOfCurrentPlaylistItem
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return nil;
 
     input_thread_t *p_input = pl_CurrentInput( p_intf );
-    if (!p_input) return nil;
+    if( !p_input )
+        return nil;
 
     input_item_t *p_item = input_GetItem( p_input );
-    if (!p_item)
+    if( !p_item )
     {
         vlc_object_release( p_input );
         return nil;
     }
 
     char *psz_uri = input_item_GetURI( p_item );
-    if (!psz_uri)
+    if( !psz_uri )
     {
         vlc_object_release( p_input );
         return nil;
@@ -306,9 +311,9 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
     NSURL * o_url = [NSURL URLWithString: [NSString stringWithUTF8String: psz_uri]];
     free( psz_uri );
 
-    if ([o_name isEqualToString:@""])
+    if( [o_name isEqualToString:@""] )
     {
-        if ([o_url isFileURL]) 
+        if( [o_url isFileURL] ) 
             o_name = [[NSFileManager defaultManager] displayNameAtPath: [o_url path]];
         else
             o_name = [o_url absoluteString];
@@ -372,7 +377,7 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
 - (void)shuffle
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return;
 
     vlc_value_t val;
@@ -384,7 +389,7 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
     var_Set( p_playlist, "random", val );
     if( val.b_bool )
     {
-        if (p_vout)
+        if( p_vout )
         {
             vout_OSDMessage( p_vout, SPU_DEFAULT_CHANNEL, "%s", _( "Random On" ) );
             vlc_object_release( p_vout );
@@ -393,7 +398,7 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
     }
     else
     {
-        if (p_vout)
+        if( p_vout )
         {
             vout_OSDMessage( p_vout, SPU_DEFAULT_CHANNEL, "%s", _( "Random Off" ) );
             vlc_object_release( p_vout );
@@ -405,7 +410,7 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
 - (void)repeatAll
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return;
 
     playlist_t * p_playlist = pl_Get( p_intf );
@@ -416,7 +421,7 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
     config_PutInt( p_playlist, "loop", YES );
 
     vout_thread_t *p_vout = getVout();
-    if (p_vout)
+    if( p_vout )
     {
         vout_OSDMessage( p_vout, SPU_DEFAULT_CHANNEL, "%s", _( "Repeat All" ) );
         vlc_object_release( p_vout );
@@ -426,7 +431,7 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
 - (void)repeatOne
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return;
 
     playlist_t * p_playlist = pl_Get( p_intf );
@@ -437,7 +442,7 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
     config_PutInt( p_playlist, "loop", NO );
 
     vout_thread_t *p_vout = getVout();
-    if (p_vout)
+    if( p_vout )
     {
         vout_OSDMessage( p_vout, SPU_DEFAULT_CHANNEL, "%s", _( "Repeat One" ) );
         vlc_object_release( p_vout );
@@ -447,7 +452,7 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
 - (void)repeatOff
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return;
 
     playlist_t * p_playlist = pl_Get( p_intf );
@@ -458,72 +463,44 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
     config_PutInt( p_playlist, "loop", NO );
 
     vout_thread_t *p_vout = getVout();
-    if (p_vout)
+    if( p_vout )
     {
         vout_OSDMessage( p_vout, SPU_DEFAULT_CHANNEL, "%s", _( "Repeat Off" ) );
         vlc_object_release( p_vout );
     }
 }
 
-- (void)displayVolume
-{
-    vout_thread_t *p_vout = getVout();
-    if (p_vout)
-    {
-        vout_OSDSlider( p_vout, SPU_DEFAULT_CHANNEL,
-                       [self volume]*100/AOUT_VOLUME_MAX, OSD_VERT_SLIDER );
-        vout_OSDMessage( p_vout, SPU_DEFAULT_CHANNEL, _( "Volume %d%%" ),
-                       [self volume]*100/AOUT_VOLUME_DEFAULT );
-        vlc_object_release( p_vout );
-    }
-}
-
 - (void)volumeUp
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return;
 
     aout_VolumeUp( pl_Get( p_intf ), 1, NULL );
-    [self displayVolume];
 }
 
 - (void)volumeDown
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return;
 
     aout_VolumeDown( pl_Get( p_intf ), 1, NULL );
-    [self displayVolume];
 }
 
 - (void)mute
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return;
 
     aout_ToggleMute( pl_Get( p_intf ), NULL );
-
-    vout_thread_t *p_vout = getVout();
-    if( p_vout )
-    {
-        if( [self isMuted] )
-        {
-            vout_OSDIcon( p_vout, SPU_DEFAULT_CHANNEL, OSD_MUTE_ICON );
-        }
-        else
-            [self displayVolume];
-
-        vlc_object_release( p_vout );
-    }
 }
 
 - (BOOL)isMuted
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return NO;
 
     BOOL b_is_muted = NO;
@@ -535,7 +512,7 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
 - (int)volume
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return 0;
 
     audio_volume_t i_volume = aout_VolumeGet( pl_Get( p_intf ) );
@@ -546,32 +523,130 @@ static VLCCoreInteraction *_o_sharedInstance = nil;
 - (void)setVolume: (int)i_value
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return;
 
     aout_VolumeSet( pl_Get( p_intf ), i_value );
 }
 
+#pragma mark -
+#pragma mark drag and drop support for VLCVoutView, VLBrushedMetalImageView and VLCThreePartDropView
+- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
+{
+    NSPasteboard *o_paste = [sender draggingPasteboard];
+    NSArray *o_types = [NSArray arrayWithObject: NSFilenamesPboardType];
+    NSString *o_desired_type = [o_paste availableTypeFromArray:o_types];
+    NSData *o_carried_data = [o_paste dataForType:o_desired_type];
+    BOOL b_autoplay = config_GetInt( VLCIntf, "macosx-autoplay" );
+
+    if( o_carried_data )
+    {
+        if( [o_desired_type isEqualToString:NSFilenamesPboardType] )
+        {
+            NSArray *o_array = [NSArray array];
+            NSArray *o_values = [[o_paste propertyListForType: NSFilenamesPboardType] sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
+            NSUInteger count = [o_values count];
+
+            input_thread_t * p_input = pl_CurrentInput( VLCIntf );
+            BOOL b_returned = NO;
+
+            if( count == 1 && p_input )
+            {
+                b_returned = input_AddSubtitle( p_input, make_URI([[o_values objectAtIndex:0] UTF8String], NULL), true );
+                vlc_object_release( p_input );
+                if( !b_returned )
+                    return YES;
+            }
+            else if( p_input )
+                vlc_object_release( p_input );
+
+            for( NSUInteger i = 0; i < count; i++)
+            {
+                NSDictionary *o_dic;
+                char *psz_uri = make_URI([[o_values objectAtIndex:i] UTF8String], NULL);
+                if( !psz_uri )
+                    continue;
+
+                o_dic = [NSDictionary dictionaryWithObject:[NSString stringWithCString:psz_uri encoding:NSUTF8StringEncoding] forKey:@"ITEM_URL"];
+                free( psz_uri );
+
+                o_array = [o_array arrayByAddingObject: o_dic];
+            }
+            if( b_autoplay )
+                [[[VLCMain sharedInstance] playlist] appendArray: o_array atPos: -1 enqueue:NO];
+            else
+                [[[VLCMain sharedInstance] playlist] appendArray: o_array atPos: -1 enqueue:YES];
+
+            return YES;
+        }
+    }
+    return NO;
+}
+
 #pragma mark -
 #pragma mark video output stuff
 
 - (void)setAspectRatioLocked:(BOOL)b_value
 {
-    b_lockAspectRatio = b_value;
+    config_PutInt( VLCIntf, "macosx-lock-aspect-ratio", b_value );
 }
 
 - (BOOL)aspectRatioIsLocked
 {
-    return b_lockAspectRatio;
+    return config_GetInt( VLCIntf, "macosx-lock-aspect-ratio" );
 }
 
 - (void)toggleFullscreen
 {
     intf_thread_t *p_intf = VLCIntf;
-    if (!p_intf)
+    if( !p_intf )
         return;
 
-    var_ToggleBool( pl_Get( p_intf ), "fullscreen" );
+    BOOL b_fs = var_ToggleBool( pl_Get( p_intf ), "fullscreen" );
+
+    vout_thread_t *p_vout = getVout();
+    if( p_vout )
+    {
+        var_SetBool( p_vout, "fullscreen", b_fs );
+        vlc_object_release( p_vout );
+    }
+}
+
+#pragma mark -
+#pragma mark uncommon stuff
+
+- (BOOL)fixPreferences
+{
+    NSMutableString * o_workString;
+    NSRange returnedRange;
+    NSRange fullRange;
+    BOOL b_needsRestart = NO;
+
+    #define fixpref( pref ) \
+    o_workString = [[NSMutableString alloc] initWithFormat:@"%s", config_GetPsz( VLCIntf, pref )]; \
+    if ([o_workString length] > 0) \
+    { \
+        returnedRange = [o_workString rangeOfString:@"macosx" options: NSCaseInsensitiveSearch]; \
+        if (returnedRange.location != NSNotFound) \
+        { \
+            if ([o_workString isEqualToString:@"macosx"]) \
+                [o_workString setString:@""]; \
+            fullRange = NSMakeRange( 0, [o_workString length] ); \
+            [o_workString replaceOccurrencesOfString:@":macosx" withString:@"" options: NSCaseInsensitiveSearch range: fullRange]; \
+            fullRange = NSMakeRange( 0, [o_workString length] ); \
+            [o_workString replaceOccurrencesOfString:@"macosx:" withString:@"" options: NSCaseInsensitiveSearch range: fullRange]; \
+            \
+            config_PutPsz( VLCIntf, pref, [o_workString UTF8String] ); \
+            b_needsRestart = YES; \
+        } \
+    } \
+    [o_workString release]
+
+    fixpref( "control" );
+    fixpref( "extraintf" );
+    #undef fixpref
+
+    return b_needsRestart;
 }
 
 @end
