@@ -2,7 +2,7 @@
  * misc.h: code not specific to vlc
  *****************************************************************************
  * Copyright (C) 2003-2011 VLC authors and VideoLAN
- * $Id: 3d5e00d2cab7cc90641e332b7f89988bea239100 $
+ * $Id: 3540bac18215f7d31c827e51a23c9ffd8861e590 $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Felix Paul Kühne <fkuehne at videolan dot org>
@@ -23,7 +23,6 @@
  *****************************************************************************/
 
 #import <Cocoa/Cocoa.h>
-#import <ApplicationServices/ApplicationServices.h>
 #import "CompatibilityFixes.h"
 
 /*****************************************************************************
@@ -61,12 +60,16 @@
 {
     BOOL b_canBecomeKeyWindow;
     BOOL b_isset_canBecomeKeyWindow;
+    BOOL b_canBecomeMainWindow;
+    BOOL b_isset_canBecomeMainWindow;
     BOOL b_isFullscreen;
     NSViewAnimation *animation;
 }
 
 - (void)setCanBecomeKeyWindow: (BOOL)canBecomeKey;
 
+- (void)setCanBecomeMainWindow: (BOOL)canBecomeMain;
+
 /* animate mode is only supported in >=10.4 */
 - (void)orderFront: (id)sender animate: (BOOL)animate;
 
@@ -85,16 +88,6 @@
 @end
 
 
-/*****************************************************************************
- * VLCControllerView
- *****************************************************************************/
-
-@interface VLCControllerView : NSView
-{
-}
-
-@end
-
 /*****************************************************************************
  * VLBrushedMetalImageView
  *****************************************************************************/
@@ -187,7 +180,7 @@
 @end
 
 /*****************************************************************************
- * VLCThreePartImageView interface
+ * VLCThreePartDropView interface
  *****************************************************************************/
 @interface VLCThreePartDropView : VLCThreePartImageView
 {
