@@ -1,8 +1,8 @@
 /*****************************************************************************
  * open.h: Open dialogues for VLC's MacOS X port
  *****************************************************************************
- * Copyright (C) 2002-2009 the VideoLAN team
- * $Id: e2080c22a911e8e0a8d9c642c67309c7f55910ba $
+ * Copyright (C) 2002-2011 VLC authors and VideoLAN
+ * $Id: 73c8d1b5a72f44dcd1ecfa572bba17aab09918fa $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -24,15 +24,20 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
-NSArray *GetEjectableMediaOfClass( const char *psz_class );
+#define kVLCMediaAudioCD "AudioCD"
+#define kVLCMediaDVD "DVD"
+#define kVLCMediaVCD "VCD"
+#define kVLCMediaSVCD "SVCD"
+#define kVLCMediaBD "Bluray"
+#define kVLCMediaVideoTSFolder "VIDEO_TS"
+#define kVLCMediaBDMVFolder "BDMV"
+#define kVLCMediaUnknown "Unknown"
 
 /*****************************************************************************
  * Intf_Open interface
  *****************************************************************************/
 @interface VLCOpen : NSObject
 {
-    IBOutlet id o_playlist;
-
     IBOutlet id o_panel;
 
     IBOutlet id o_mrl_fld;
@@ -49,26 +54,65 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
     IBOutlet id o_sout_options;
 
     /* open file */
-    IBOutlet id o_file_path;
+    IBOutlet id o_file_name;
+    IBOutlet id o_file_name_stub;
+    IBOutlet id o_file_icon_well;
     IBOutlet id o_file_btn_browse;
     IBOutlet id o_file_stream;
     IBOutlet id o_file_slave_ckbox;
     IBOutlet id o_file_slave_select_btn;
-    IBOutlet id o_file_slave_filename_txt;
+    IBOutlet id o_file_slave_filename_lbl;
+    IBOutlet id o_file_slave_icon_well;
+    IBOutlet id o_file_subtitles_filename_lbl;
+    IBOutlet id o_file_subtitles_icon_well;
 
     /* open disc */
-    IBOutlet id o_disc_type;
-    IBOutlet id o_disc_device;
-    IBOutlet id o_disc_device_lbl;
-    IBOutlet id o_disc_title;
-    IBOutlet id o_disc_title_lbl;
-    IBOutlet id o_disc_title_stp;
-    IBOutlet id o_disc_chapter;
-    IBOutlet id o_disc_chapter_lbl;
-    IBOutlet id o_disc_chapter_stp;
-    IBOutlet id o_disc_videots_folder;
-    IBOutlet id o_disc_videots_btn_browse;
-    IBOutlet id o_disc_dvd_menus;
+    IBOutlet id o_disc_selector_pop;
+
+    IBOutlet id o_disc_nodisc_view;
+    IBOutlet id o_disc_nodisc_lbl;
+    IBOutlet id o_disc_nodisc_videots_btn;
+    IBOutlet id o_disc_nodisc_bdmv_btn;
+
+    IBOutlet id o_disc_audiocd_view;
+    IBOutlet id o_disc_audiocd_lbl;
+    IBOutlet id o_disc_audiocd_trackcount_lbl;
+    IBOutlet id o_disc_audiocd_videots_btn;
+    IBOutlet id o_disc_audiocd_bdmv_btn;
+
+    IBOutlet id o_disc_dvd_view;
+    IBOutlet id o_disc_dvd_lbl;
+    IBOutlet id o_disc_dvd_disablemenus_btn;
+    IBOutlet id o_disc_dvd_videots_btn;
+    IBOutlet id o_disc_dvd_bdmv_btn;
+
+    IBOutlet id o_disc_dvdwomenus_view;
+    IBOutlet id o_disc_dvdwomenus_lbl;
+    IBOutlet id o_disc_dvdwomenus_enablemenus_btn;
+    IBOutlet id o_disc_dvdwomenus_videots_btn;
+    IBOutlet id o_disc_dvdwomenus_bdmv_btn;
+    IBOutlet id o_disc_dvdwomenus_title;
+    IBOutlet id o_disc_dvdwomenus_title_lbl;
+    IBOutlet id o_disc_dvdwomenus_title_stp;
+    IBOutlet id o_disc_dvdwomenus_chapter;
+    IBOutlet id o_disc_dvdwomenus_chapter_lbl;
+    IBOutlet id o_disc_dvdwomenus_chapter_stp;
+
+    IBOutlet id o_disc_vcd_view;
+    IBOutlet id o_disc_vcd_lbl;
+    IBOutlet id o_disc_vcd_videots_btn;
+    IBOutlet id o_disc_vcd_bdmv_btn;
+    IBOutlet id o_disc_vcd_title;
+    IBOutlet id o_disc_vcd_title_lbl;
+    IBOutlet id o_disc_vcd_title_stp;
+    IBOutlet id o_disc_vcd_chapter;
+    IBOutlet id o_disc_vcd_chapter_lbl;
+    IBOutlet id o_disc_vcd_chapter_stp;
+
+    IBOutlet id o_disc_bd_view;
+    IBOutlet id o_disc_bd_lbl;
+    IBOutlet id o_disc_bd_videots_btn;
+    IBOutlet id o_disc_bd_bdmv_btn;
 
     /* open network */
     IBOutlet id o_net_http_url;
@@ -99,7 +143,9 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
     IBOutlet id o_file_sub_ckbox;
     IBOutlet id o_file_sub_btn_settings;
     IBOutlet id o_file_sub_sheet;
-    IBOutlet id o_file_sub_path;
+    IBOutlet id o_file_sub_path_lbl;
+    IBOutlet id o_file_sub_path_fld;
+    IBOutlet id o_file_sub_icon_view;
     IBOutlet id o_file_sub_btn_browse;
     IBOutlet id o_file_sub_override;
     IBOutlet id o_file_sub_delay;
@@ -140,7 +186,6 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
 
     /* screen support */
     IBOutlet id o_screen_view;
-    IBOutlet id o_screen_lbl;
     IBOutlet id o_screen_long_lbl;
     IBOutlet id o_screen_fps_fld;
     IBOutlet id o_screen_fps_lbl;
@@ -159,9 +204,31 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
     IBOutlet id o_screen_height_stp;
     IBOutlet id o_screen_follow_mouse_ckb;
 
+    /* QTK support */
+    IBOutlet id o_qtk_view;
+    IBOutlet id o_qtk_long_lbl;
+    IBOutlet id o_qtk_device_pop;
+    IBOutlet id o_qtk_label_view;
+    IBOutlet id o_capture_width_lbl;
+    IBOutlet id o_capture_width_fld;
+    IBOutlet id o_capture_width_stp;
+    IBOutlet id o_capture_height_lbl;
+    IBOutlet id o_capture_height_fld;
+    IBOutlet id o_capture_height_stp;
+
+    NSArray         *qtkvideoDevices;
+    NSString        *qtk_currdevice_uid;
+
     BOOL b_autoplay;
+    BOOL b_nodvdmenus;
+    id o_currentOpticalMediaView;
+    id o_currentOpticalMediaIconView;
+    NSMutableArray *o_opticalDevices;
+    NSMutableArray *o_specialMediaFolders;
+    NSString *o_file_path;
     id o_currentCaptureView;
     NSString *o_file_slave_path;
+    NSString *o_sub_path;
     NSString *o_mrl;
     intf_thread_t * p_intf;
 }
@@ -171,6 +238,9 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
 - (void)setMRL:(NSString *)mrl;
 - (NSString *)MRL;
 
+- (NSArray *)qtkvideoDevices;
+- (void)qtkrefreshDevices;
+
 - (void)setSubPanel;
 - (void)openTarget:(int)i_type;
 - (void)tabView:(NSTabView *)o_tv didSelectTabViewItem:(NSTabViewItem *)o_tvi;
@@ -185,11 +255,13 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
 - (IBAction)openFileStreamChanged:(id)sender;
 
 - (void)openDisc;
-- (IBAction)openDiscTypeChanged:(id)sender;
-- (IBAction)openDiscStepperChanged:(id)sender;
-- (void)openDiscInfoChanged:(NSNotification *)o_notification;
-- (IBAction)openDiscMenusChanged:(id)sender;
-- (IBAction)openVTSBrowse:(id)sender;
+- (void)scanOpticalMedia:(NSNotification *)o_notification;
+- (IBAction)discSelectorChanged:(id)sender;
+- (IBAction)openSpecialMediaFolder:(id)sender;
+- (IBAction)dvdreadOptionChanged:(id)sender;
+- (IBAction)vcdOptionChanged:(id)sender;
+- (char *)getVolumeTypeFromMountPath:(NSString *)mountPath;
+- (NSString *)getBSDNodeFromMountPath:(NSString *)mountPath;
 
 - (void)openNet;
 - (IBAction)openNetModeChanged:(id)sender;
@@ -200,6 +272,7 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
 - (void)openCapture;
 - (void)showCaptureView: theView;
 - (IBAction)openCaptureModeChanged:(id)sender;
+- (IBAction)qtkChanged:(id)sender;
 - (IBAction)eyetvSwitchChannel:(id)sender;
 - (IBAction)eyetvLaunch:(id)sender;
 - (IBAction)eyetvGetPlugin:(id)sender;
