@@ -2,7 +2,7 @@
  * main_interface.cpp : Main interface
  ****************************************************************************
  * Copyright (C) 2006-2011 VideoLAN and AUTHORS
- * $Id: 1230404fe89bea23327a16802648a013b662e399 $
+ * $Id: 9a44a075b2abc20da5d3feae2d05f7d2d766ff24 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -210,7 +210,7 @@ MainInterface::MainInterface( intf_thread_t *_p_intf ) : QVLCMW( _p_intf )
             CONNECT( this, askVideoToResize( unsigned int, unsigned int ),
                      this, setVideoSize( unsigned int, unsigned int ) );
             CONNECT( videoWidget, sizeChanged( int, int ),
-                     this, resizeStack( int,  int ) );
+                     this, videoSizeChanged( int,  int ) );
         }
         CONNECT( this, askVideoSetFullScreen( bool ),
                  this, setVideoFullScreen( bool ) );
@@ -267,7 +267,7 @@ MainInterface::~MainInterface()
     if( himl )
         ImageList_Destroy( himl );
     if(p_taskbl)
-        p_taskbl->vt->Release(p_taskbl);
+        p_taskbl->Release();
     CoUninitialize();
 #endif
 
@@ -350,6 +350,8 @@ void MainInterface::recreateToolbars()
                  this, handleKeyPress( QKeyEvent * ) );
         THEMIM->requestVoutUpdate();
     }
+
+    setMinimalView( b_minimalView );
 }
 
 void MainInterface::reloadPrefs()
@@ -528,6 +530,11 @@ inline void MainInterface::showTab( QWidget *widget )
 #ifdef DEBUG_INTF
     msg_Warn( p_intf, "Old stackCentralOldWidget %i", stackCentralW->indexOf( stackCentralOldWidget ) );
 #endif
+    /* fixing when the playlist has been undocked after been hidden.
+       restoreStackOldWidget() is called when video stops but
+       stackCentralOldWidget would still be pointing to playlist */
+    if ( widget == playlistWidget && !isPlDocked() )
+        widget = bgWidget;
 
     stackCentralOldWidget = stackCentralW->currentWidget();
     stackWidgetsSizes[stackCentralOldWidget] = stackCentralW->size();
@@ -553,7 +560,9 @@ inline void MainInterface::showTab( QWidget *widget )
         /* Embedded playlist -> Non-embedded playlist */
         if( bgWidget == stackCentralOldWidget && widget == videoWidget )
         {
-            playlistWidget->artContainer->removeWidget( videoWidget );
+            /* In rare case when video is started before the interface */
+            if( playlistWidget != NULL )
+                playlistWidget->artContainer->removeWidget( videoWidget );
             videoWidget->show(); videoWidget->raise();
             stackCentralW->addWidget( videoWidget );
             stackCentralW->setCurrentWidget( videoWidget );
@@ -678,6 +687,12 @@ void MainInterface::setVideoSize( unsigned int w, unsigned int h )
         videoWidget->SetSizing( w, h );
 }
 
+void MainInterface::videoSizeChanged( int w, int h )
+{
+    if( !playlistWidget || playlistWidget->artContainer->currentWidget() != videoWidget )
+        resizeStack( w, h );
+}
+
 void MainInterface::setVideoFullScreen( bool fs )
 {
     b_videoFullScreen = fs;
@@ -1289,7 +1304,8 @@ void MainInterface::keyPressEvent( QKeyEvent *e )
 
 void MainInterface::handleKeyPress( QKeyEvent *e )
 {
-    if( ( e->modifiers() &  Qt::ControlModifier ) && ( e->key() == Qt::Key_H ) )
+    if( ( ( e->modifiers() & Qt::ControlModifier ) && ( e->key() == Qt::Key_H ) ) ||
+        ( b_minimalView && !b_videoFullScreen && e->key() == Qt::Key_Escape ) )
     {
         toggleMinimalView( !b_minimalView );
         e->accept();
